(* misc/cgi_support.ml: some OCaml code to browse demexp database *)

(* 
  copyright 2005 David MENTRE
  copyright 2005 Serge LEBLANC.
  this code is in the free domain
*)

(* to compile:
ocamlc -i -o misc/cgi_support -I net -I `ocamlfind query equeue` -I `ocamlfind query rpc` str.cma unix.cma equeue.cma rpc.cma net/messages_aux.cmo net/messages_clnt.cmo misc/cgi_support.ml *)

open Messages_aux
open Messages_clnt
open Printf

let connect_to_server host port =
  let client = Demexp.V1.create_client
      (Rpc_client.Inet (host, port))
      Rpc.Tcp in
  (* no need to login, we can stay Anonymous
  let login = "toto" in
  let pass = "toto" in
  let cookie = Demexp.V1.login client (login, pass) in
   *)
  (client, 0)

let disconnect_of_server client cookie =
  Demexp.V1.goodbye client cookie;
  Rpc_client.shut_down client

let get_tags client cookie =
  let tags = ref [] in
  let ret = Demexp.V1.max_tag_id client cookie in
  if ret.max_tag_id_rc <> rt_ok then
    failwith "get_tags: fatal error at max_tag_id";
  let number = Rtypes.int_of_uint4 max_number_ids in
  let rec get_some_tags base max_id =
    if base <= max_id then (
      let ret = Demexp.V1.tag_info client (cookie, base, number) in
      if ret.tag_info_rc <> rt_ok then
        failwith "get_tags: fatal error at get_some_tags";
      Array.iter 
        (fun elt -> tags := (elt.a_tag_id, elt.a_tag_label) :: !tags) 
        ret.tag_info;
      get_some_tags (base + number) max_id
     ) in
  get_some_tags 0 ret.max_tag_id;
  !tags

let tags_of_questions client cookie base number =
  let tags = Hashtbl.create number in
  let r = Demexp.V1.tag_set_of_question_group client (cookie, base, number) in
  if r.tag_set_group_rc <> rt_ok then
    failwith "tags_of_questions: fatal error at tag_set_of_question_group";
  Array.iter 
    (fun s -> Hashtbl.add tags s.tag_set_qid (Array.to_list s.tag_set))
    r.tag_set_group;
  tags




let get_questions client cookie =
  let quest_id = 1 in
  let question_filter q = 
      if q.q_id = quest_id then q.q_desc else q.q_desc in
    let cons_questions_desc l f b n =
      (*let a = Array.init n (function x -> Printf.sprintf "(base: %d, number: %d, elem: %d)" b n (x+b)) in
        l @ (Array.to_list a) in
      *)
      let ret = Demexp.V1.question_info client (cookie,b,n) in
        if ret.question_info_rc <> rt_ok then
          failwith "get_questions: fatal error at question_info" ;
          l @ Array.to_list (Array.map f ret.question_info) in
      let max_number = Rtypes.int_of_uint4 max_number_ids and
          ret = Demexp.V1.max_question_id client cookie in
            if ret.max_question_id_rc <> rt_ok then
              failwith "get_questions: fatal error at max_question_id" ; 
              let rec get_some_questions l =
                let base = List.length l in
                match ret.max_question_id - base with
                  | x when x < max_number -> cons_questions_desc l question_filter base x
                  | _ -> get_some_questions (cons_questions_desc l question_filter base max_number) in
            (get_some_questions []) @ [ (Printf.sprintf "max_number: %d" max_number); (Printf.sprintf "max_question_id: %d" ret.max_question_id) ]
            (*(get_some_questions []) @ [ (Printf.sprintf "max_number: %d" max_number); (Printf.sprintf "max_question_id: %d" ret.max_question_id) ]*)


let get_questions2 client cookie =
  let questions = ref [] in
  let add_question tags q =
    if q.q_info_status = public then (
      let q_tags =
        try Hashtbl.find tags q.q_id
        with Not_found -> [] in
      questions := (q.q_id, q.q_desc, q_tags) :: !questions
     ) in
  let ret = Demexp.V1.max_question_id client cookie in
  if ret.max_question_id_rc <> rt_ok then
    failwith "get_questions: fatal error at max_question_id";
  let max_number = Rtypes.int_of_uint4 max_number_ids in
  let rec get_some_questions base max_id =
    if base <= max_id then (
      let number_to_get = 
        if max_id - base + 1 <= max_number then max_id - base + 1
        else max_number in
      let ret = 
        Demexp.V1.question_info client (cookie, base, number_to_get) in
      if ret.question_info_rc <> rt_ok then
        failwith "get_questions: fatal error at question_info";
      let tags =
        tags_of_questions client cookie base number_to_get in
      Array.iter (add_question tags) ret.question_info;
      get_some_questions (base + number_to_get) max_id
     ) in
  get_some_questions 0 ret.max_question_id;
  !questions
  
(*
let get_question_number number client cookie =
  let ret = Demexp.V1.question_info client (cookie,number,1) in
  if ret.question_info_rc <> rt_ok then
    failwith "get_questions: fatal error at question_info"
  else  
    ret.question_info.(0).q_desc
  ;;
*)

let parse_url ~url =
  ignore(Str.string_match (Str.regexp "^\\([^:]+\\):\\(.+\\)$") url 0);
  let host = Str.matched_group 1 url in
  let port = int_of_string (Str.matched_group 2 url) in
  host, port

let request_tags ~url =
  let host, port = parse_url ~url in
  let client, cookie = connect_to_server host port in
  let tags = get_tags client cookie in
  disconnect_of_server client cookie;
  tags

let request_questions ~url =
  let host, port = parse_url ~url in
  let client, cookie = connect_to_server host port in
  let questions = get_questions client cookie in
  disconnect_of_server client cookie;
  questions
 
let request_questions2 ~url =
  let host, port = parse_url ~url in
  let client, cookie = connect_to_server host port in
  let questions = get_questions2 client cookie in
  disconnect_of_server client cookie;
  questions

let string_of_int_list l =
  (List.fold_left (fun str i -> str ^ (string_of_int i) ^ "; ") "[" l) ^ "]"

(*
let _ =
  let url = "tuxinette.linux-france.org:50000" in
  let print_tags (id, str) = printf "%d:%s\n" id str in
  printf "** tags **\n";
  List.iter print_tags (request_tags ~url);

  let print_questions (id, str, tags) =
    printf "%d:%s\n\t %s\n" id str (string_of_int_list tags) in
    
  printf "** questions **\n";
  List.iter print_questions (request_questions2 ~url)
*)    
